'''
run point of application. Calls films to cal module.
Will soon allow configuration and downloading of watchlist.
'''

from film_to_cal import film_to_cal

if __name__ == "__main__":
    # film_to_calc
    film_to_cal()

    # TODO #

    # add favorite theater

    # update watchlist using letterboxd

    # set busy schedule

    # keep track of films already scheduled and do not promptif sceance has passed
